@@ -52,7 +52,7 @@
 
 static void g1_debug(const char *str, void *data)
 {
-	DBG("%s", str);
+	ofono_info("%s", str);
 }
 
 /* Detect hardware, and initialize if found */
@@ -168,7 +168,7 @@ static void g1_post_sim(struct ofono_modem *modem)
 	ofono_call_meter_create(modem, 0, "atmodem", chat);
 	ofono_call_barring_create(modem, 0, "atmodem", chat);
 	ofono_ssn_create(modem, 0, "atmodem", chat);
-	ofono_sms_create(modem, OFONO_VENDOR_HTC_G1, "atmodem", chat);
+	ofono_sms_create(modem, OFONO_VENDOR_QUALCOMM_MSM, "atmodem", chat);
 	ofono_phonebook_create(modem, 0, "atmodem", chat);
 
 	mw = ofono_message_waiting_create(modem);
