@@ -41,7 +41,7 @@
 #define uninitialized_var(x) x = x
 
 #define SMS_BACKUP_MODE 0600
-#define SMS_BACKUP_PATH STORAGEDIR "/%s/sms"
+#define SMS_BACKUP_PATH STORAGEDIR "/%s/sms_assembly"
 #define SMS_BACKUP_PATH_DIR SMS_BACKUP_PATH "/%s-%i-%i"
 #define SMS_BACKUP_PATH_FILE SMS_BACKUP_PATH_DIR "/%03i"
 
@@ -805,7 +805,7 @@ static gboolean decode_submit_report(const unsigned char *pdu, int len,
 	int offset = 0;
 	unsigned char octet;
 	gboolean udhi;
-	guint8 fcs;
+	guint8 uninitialized_var(fcs);
 	guint8 pi;
 	struct sms_scts *scts;
 	guint8 pid = 0;
@@ -1096,7 +1096,7 @@ static gboolean decode_deliver_report(const unsigned char *pdu, int len,
 	int offset = 0;
 	unsigned char octet;
 	gboolean udhi;
-	guint8 fcs;
+	guint8 uninitialized_var(fcs);
 	guint8 pi;
 	guint8 pid = 0;
 	guint8 dcs = 0;
@@ -2133,6 +2133,9 @@ char *sms_decode_text(GSList *sms_list)
 
 		udl_in_bytes = sms_udl_in_bytes(udl, dcs);
 
+		if (udl_in_bytes == taken)
+			continue;
+
 		if (charset == SMS_CHARSET_7BIT) {
 			unsigned char buf[160];
 			long written;
@@ -2187,11 +2190,6 @@ char *sms_decode_text(GSList *sms_list)
 
 	utf8 = g_string_free(str, FALSE);
 
-	if (strlen(utf8) == 0) {
-		g_free(utf8);
-		return NULL;
-	}
-
 	return utf8;
 }
 
